----------------------------------------------------------------------
-- CruiseControl v1.5.60100
-- Author: Aryl
-- last change by Aryl on 2015-02-26T22:42:00Z
-- for 60100

----------------------------------------------------------------------
-- Localisation
local L = CruiseControlLocale;

----------------------------------------------------------------------
-- print routines
--
local HEADER = "|cFFccb938CruiseControl:|r ";

local function msg(s)
      DEFAULT_CHAT_FRAME:AddMessage(HEADER..s);
end

local function debug(s)
      --[===[@debug@
      DEFAULT_CHAT_FRAME:AddMessage(HEADER..s);
      --@end-debug@]===]
end

local function error(s)
      DEFAULT_CHAT_FRAME:AddMessage(HEADER.."|cFFcf682fERROR:|r "..s);
end

local function say(x, ...)
  local t = L[x];
  local c = ...;
  if (not t) then
    debug("no localisation for "..x);
    t = x;
  end
  if (c) then
    msg("|c"..c..t..".|r");
  else
    msg(t);
  end
end

debug("loading... "..GetLocale());

-------------------------------------------------------------------------
-- main frame and libs
--
CruiseControl = CreateFrame("Frame", nil, WorldMapDetailFrame);

local Astrolabe = DongleStub("Astrolabe-1.0");

----------------------------------------------------------------------
-- config vars
local UPDATE_INTERVAL = 0.175; -- seconds
local SOUND_INTERVAL = 1; -- seconds
local APPROACH_LIMIT = 11; -- seconds
local LINEWIDTH = 48;
local LINECOLOR = {0.5, 0.7, 0.8, 0.7};
local RINGWIDTH = 24;
local RINGCOLOR = {0.9, 0.9, 0.9, 1};
local FONTSIZE = 12;
local FONTSTYLE = "Fonts\\FRIZQT__.TTF";
local FONTOUTLINE = "OUTLINE";
local TARGETICONFILE = "Interface\\AddOns\\CruiseControl\\ring.tga";

local ANCHOR = {
      "ANCHOR_BOTTOMRIGHT", "ANCHOR_BOTTOMRIGHT", "ANCHOR_RIGHT", "ANCHOR_RIGHT",
};
----------------------------------------------------------------------
-- sound settings

local SOUNDSETTINGS = {
  ["Sound_MasterVolume"] = 1.0,
  ["Sound_SFXVolume"] = 0.0,
  ["Sound_MusicVolume"] = 0.0,
  ["Sound_AmbienceVolume"] = 0.0,
  ["Sound_EnableSoundWhenGameIsInBG"] = 1.0,
};

----------------------------------------------------------------------
-- sound styles
local SOUNDSTYLES = {
  ["external"] = {  
    ["DESTSET"]    = "Interface\\AddOns\\CruiseControl\\engage.mp3",
    ["DESTCLEAR"]  = "Interface\\AddOns\\CruiseControl\\disengage.mp3",
    ["DESTCLEAR2"] = "Interface\\AddOns\\CruiseControl\\disengage2.mp3",
    ["APPROACH"]   = "Interface\\AddOns\\CruiseControl\\approach.mp3",
    ["OVERSHOT"]   = "Interface\\AddOns\\CruiseControl\\overshot.mp3",
  },
  ["default"] = {
    ["DESTSET"]    = "Sound\\INTERFACE\\PickUp\\PutDownRing.ogg",
    ["DESTCLEAR"]  = "Sound\\INTERFACE\\uEscapeScreenClose.ogg", 
    ["DESTCLEAR2"]  = "Sound\\INTERFACE\\uEscapeScreenClose.ogg", 
--    ["DESTCLEAR2"] = "Sound\\Doodad\\DropOnGround.ogg",
    ["APPROACH"]   = "Sound\\INTERFACE\\AuctionWindowOpen.ogg", 
    ["OVERSHOT"]   = "Sound\\Creature\\Monkey\\MonkeyWound_01.ogg",
  },
  ["chicken"] = {
    ["DESTSET"]    = "Sound\\Creature\\BushChicken\\BushChicken_Wound_02.ogg",
    ["DESTCLEAR"]  = "Sound\\Creature\\BushChicken\\BushChicken_Wound_09.ogg",
    ["DESTCLEAR2"] = "Sound\\Creature\\BushChicken\\BushChicken_PreAggro_07.ogg",
    ["APPROACH"]   = "Sound\\Creature\\BushChicken\\BushChicken_Attack_10.ogg",
    ["OVERSHOT"]   = "Sound\\Creature\\ChickenMount\\ChickenMountWound2.ogg",
  },
  ["animals"] = {
    ["DESTSET"]    = "Sound\\Creature\\Monkey\\MonkeyAttack_03.ogg",
    ["DESTCLEAR2"] = "Sound\\Creature\\Gorilla\\GorillaAttackA.ogg",
    ["DESTCLEAR"]  = "Sound\\Creature\\Seagull\\Seagull_PreAggro_04.ogg",
    ["APPROACH"]   = "Sound\\Creature\\PugDog\\PugDog_Clickable_06.ogg",
    ["OVERSHOT"]   = "Sound\\Creature\\Seagull\\Seagull_Wound_07.ogg",
  },
};

local function playMySound(type)
  PlaySoundFile(SOUNDSTYLES[CruiseControlOptions.SoundStyle][type], "Master");
end

--    ["DESTCLEAR2"] = "Sound\\Doodad\\DR_MachineParts01_Open.ogg", -- DropOnGround.ogg";
--local DESTSET   = "Sound\\INTERFACE\\uMiniMapZoom.ogg";
--local APPROACH  = "Sound\\Item\\Weapons\\ParrySounds\\1hParryMetalHitMetal.ogg";
--local APPROACH  = "Sound\\Spells\\Tradeskills\\BlackSmithCraftingD.ogg";
--local OVERSHOT  = "Sound\\Creature\\MobileAlertBot\\MobileAlertBotWoundA.ogg"; 
--local OVERSHOT  = "Sound\\Creature\\Monkey\\Monkey_Wound_07.ogg"; 
--local OVERSHOT  = "Sound\\Creature\\Parrot\\PArrotPreAggro.ogg";
--    ["DESTCLEAR2"] = "Sound\\Creature\\Parrot\\PArrotAggro.ogg";
--local OVERSHOT  = "Sound\\Creature\\PugDog\\PugDog_Clickable_06";
--local OVERSHOT  = "Sound\\Creature\\Seagull\\Seagull_Wound_07.ogg";
--local OVERSHOT  = "Sound\\Creature\\Seagull\\Seagull_Wound_04.ogg";
--local OVERSHOT  = "Sound\\INTERFACE\\RaidWarning.ogg";

--local SOUND = {
--      "Sound\\Doodad\\ArcaneCrystalOpen.ogg", -- crash
--      "Sound\\Doodad\\Deepholm_MercuryDrip01_01.ogg",
--      "Sound\\Doodad\\DR_MachineParts01_Open.ogg",
--      "Sound\\Doodad\\G_BarrelOpen-Chest1.ogg",
--      "Sound\\Doodad\\G_HunterTrapSpawn.ogg",
--      "Sound\\Doodad\\GnomeElevatorSmallStop.ogg",
--      "Sound\\Doodad\\GO_DeepholmFloatingRock_Whoosh_01.ogg",
--      "Sound\\Doodad\\GO_MummyHand01_04.ogg",
--      "Sound\\Doodad\\GO_MummyHand01_06.ogg",
--      "Sound\\Doodad\\Goblin_KezanBillboard_01_Lights_01.ogg",
--      "Sound\\INTERFACE\\uMiniMapZoom.ogg",
--      "Sound\\INTERFACE\\SheathMetal.ogg",
--      "Sound\\INTERFACE\\igQuestFailed.ogg",
--      "Sound\\INTERFACE\\DropOnGround.ogg",
--      "Sound\\INTERFACE\\RaidWarning.ogg",
--      "Sound\\INTERFACE\\AuctionWindowOpen.ogg", -- ding
--      "Sound\\INTERFACE\\PVPFlagTakenMono.ogg",
--      "Sound\\INTERFACE\\UI_BnetToast.ogg",  -- uinggg
--      "Sound\\INTERFACE\\AuctionWindowClose.ogg",
--      "Sound\\INTERFACE\\LFG_Denied.ogg",
--      "Sound\\INTERFACE\\PutDownRing.ogg",
--      "Sound\\Item\\1hParryMetalHitMetal.ogg",

--      "Sound\Spells\SPELL_PoppingCorn_Cast_01.ogg", -- poff disengage
--      "Sound\Spells\IceCrown_Bug_Attack_07.ogg", -- tick engage
--      "Sound\Spells\Bonk2.ogg", -- disengage
--      "Sound\Spells\Spell_Uni_CameraFlash_04.ogg", -- disengage
--      "Sound\Spells\Spell_Uni_CameraShutter_01.ogg", -- engage
--      "Sound\Spells\PetCall.ogg", -- engage
--      "Sound\Spells\Spell_Uni_Glass_Bottle_Break_08.ogg", -- disengage
--      "Sound\Spells\ShieldBreak.ogg", -- disengage2
--      "Sound\Spells\FluteRun.ogg", -- approach
--      "Sound\Spells\SPELL_ShootTechGun_Cast_01.ogg", -- approach, overshot
--      "Sound\Spells\Fart_02.ogg", -- 
--      "Sound\Spells\ThunderClap.ogg", -- 
--      "Sound\Spells\SpiritWolf.ogg", -- disengage
--      "Sound\Spells\SPELL_ShootTechGun_Cast_03.ogg", -- approach, overshot
--      "Sound\Spells\Spell_Uni_SonarPing_01.ogg", -- engage
--      "Sound\Spells\KnockDown.ogg", -- disengage
--      "Sound\Universal\FM_Takeoff.ogg", -- engage
--      "Sound\Universal\FM_Land.ogg", -- disengage2
--      "Sound\VEHICLES\MotorcycleVehicle\MotorcycleVehicleJumpStart1.ogg", -- engage
--      "Sound\VEHICLES\MotorcycleVehicle\MotorcycleVehicleJumpEnd3.ogg", -- disengage
--      "Sound\Item\Weapons\Whip\BullWhipHit2.ogg", -- approach
--      "Sound\Spells\", -- 
--      "Sound\Spells\", -- 
--      "Sound\Spells\", -- 
--      "Sound\Spells\", -- 

--};

----------------------------------------------------------------------
-- slash command handling
SLASH_CruiseControl1 = "/CruiseControl";
SLASH_CruiseControl2 = "/CC";
function SlashCmdList.CruiseControl(arg, editbox)
  local command, rest = arg:match("^(%S*)%s*(.-)$");
  if command == "soundstyle" then
    if SOUNDSTYLES[rest] then
      CruiseControlOptions.SoundStyle = rest;
      msg(L["soundstyle set to "]..rest);
    else
      msg(L["unknown soundstyle: "]..rest);
      say("known soundstyles: 'default', 'external' or 'animals'");
    end
  else
    say("Syntax: /CruiseControl soundstyle <style>");
    say("where <style> is one of 'default', 'external' or 'animals'");
  end
end

----------------------------------------------------------------------
-- global vars
local PlayerX = 0;
local PlayerY = 0;
local Angle = 0;
local MouseX = nil;
local MouseY = nil;

local Prev = {};
Prev.C, Prev.Z, Prev.X, Prev.Y = Astrolabe:GetCurrentPlayerPosition();
local MaxSpeed = 7;
local From = {};
From.TOA = 0;

--------------------------------------------------------------------
-- GetCurrentCursorPosition
--
local function GetCurrentCursorPosition()
	local x, y = GetCursorPosition()
	local left, top = WorldMapDetailFrame:GetLeft(), WorldMapDetailFrame:GetTop()
	local width = WorldMapDetailFrame:GetWidth()
	local height = WorldMapDetailFrame:GetHeight()
	local scale = WorldMapDetailFrame:GetEffectiveScale()
	local cx = (x/scale - left) / width
	local cy = (top - y/scale) / height

	if cx < 0 or cx > 1 or cy < 0 or cy > 1 then
		return nil, nil
	end
	return cx, cy
end



-------------------------------------------------------------------
-- Init
--
function CruiseControl:Init()
  self.timeSinceLastUpdate = 0;
  self.timeSinceLastSound = 0;
  self.currentUnit = "player";
  self.targetSet = false;
  self.startedFlying = false;
  self.targetReached = false;
  self.approachAnnounced = false;

  if CruiseControlOptions == nil then
    CruiseControlOptions = {};
    CruiseControlOptions.SoundStyle = 'default';
    debug("setting sound to default");
  else
    if not SOUNDSTYLES[CruiseControlOptions.SoundStyle] then
      CruiseControlOptions.SoundStyle = "default";
      debug("resetting sound to default");
    end
    debug("sound style is "..CruiseControlOptions.SoundStyle);
  end

  -- text display on world map
  self.textString = WorldMapButton:CreateFontString(nil, "OVERLAY");
  self.textString:SetTextColor(1, 1, 1, 1);
  self.textString:SetPoint("BOTTOMLEFT", WorldMapButton, "BOTTOMLEFT", 3, 3);

  self:SetWidth(self:GetParent():GetWidth());
  self:SetHeight(self:GetParent():GetHeight());
  self:SetPoint("TOPLEFT");
  self:EnableMouse(true);
  self:Show();

  -- 
  self.approachFrame = CreateFrame("Frame", "CC_Approach", UIParent);
  self.approachFrame.timeSinceLastSound = 0;
  self.approachFrame:SetScript("OnUpdate", function (self, elapsed)
    self.timeSinceLastSound = self.timeSinceLastSound + elapsed;
    if self.timeSinceLastSound < SOUND_INTERVAL then
      return;
    end
    local tsls = self.timeSinceLastSound;
    self.timeSinceLastSound = 0;
    if not CruiseControl.targetSet then
      Prev.C, Prev.Z, Prev.X, Prev.Y = Astrolabe:GetCurrentPlayerPosition();
      --Prev.takenAt = GetTime();
      return;
    end
    local c, z, x, y = Astrolabe:GetCurrentPlayerPosition();
    if not c or not z then
      -- cannot get current player position, skip this round
      return;
    end

    local dist = Astrolabe:ComputeDistance(Prev.C, Prev.Z, Prev.X, Prev.Y, c, z, x, y);
    --local diff = GetTime() - Prev.takenAt;
    local speed = 0;
    if dist then
      speed = dist / tsls;
    end
    if speed > 0 then
      CruiseControl.startedFlying = true;
    else
      -- we stopped, disengage
      if CruiseControl.startedFlying then
        say("disengaged", "FF7091f2");
        if CruiseControl.targetReached then
          playMySound("DESTCLEAR2");
	else
          playMySound("DESTCLEAR");
	end
        if CruiseControl.soundSettings then
          for k, v in pairs(CruiseControl.soundSettings) do
	    SetCVar(k, v);
	  end
          CruiseControl.soundSettings = nil;
        end
        CruiseControl.targetSet = false;	
        CruiseControl.startedFlying = false;
	CruiseControl.targetReached = false;
	CruiseControl.approachAnnounced = false;
      end
    end
    Prev.C, Prev.Z, Prev.X, Prev.Y = c, z, x, y; --Astrolabe:GetCurrentPlayerPosition();
    Prev.takenAt = GetTime();
    if CruiseControl.targetSet then
      dist = From.Distance - Astrolabe:ComputeDistance(From.C, From.Z, From.X, From.Y, c, z, x, y);
      if speed > 0 then
        From.TOA = dist / speed;
      else
        From.TOA = 0;
      end
      if dist < speed * SOUND_INTERVAL then
        if not CruiseControl.targetReached then
          say("destination reached", "FFdf772f");
          CruiseControl.targetReached = true;
        end
	playMySound("OVERSHOT");
      else
        if dist < speed * APPROACH_LIMIT then
	  if not CruiseControl.approachAnnounced then
            say("approaching destination");
	    CruiseControl.approachAnnounced = true;
	    -- save sound settings and set to loud
            if not CruiseControl.soundSettings then
	      CruiseControl.soundSettings = {};
	      for k, v in pairs(SOUNDSETTINGS) do
	        CruiseControl.soundSettings[k] = GetCVar(k);
	        SetCVar(k, v);
	      end
            end
	  end
	  playMySound("APPROACH");
	end
      end
    end
  end
  );

  self.target = CreateFrame("Button", "CC_Target", self);
  self.target.quadrantCode = 1;
  self.target:SetSize(RINGWIDTH, RINGWIDTH);
  self.targetTexture = self.target:CreateTexture("CC_TargetTexture", "OVERLAY");
  if not self.targetTexture:SetTexture(TARGETICONFILE) then
    error("Cannot load ring.tga");
  end;
  self.targetTexture:SetParent(self.target);
  self.targetTexture:SetAllPoints();
  self.targetTexture:SetSize(RINGWIDTH, RINGWIDTH);
  self.targetTexture:SetVertexColor(RINGCOLOR[0], RINGCOLOR[1], RINGCOLOR[2], RINGCOLOR[3]);
  self.targetTexture:Show();
  self.target:SetFrameStrata("HIGH");
  self.target:EnableMouse(true);

  self.target:SetScript("OnEnter", function (self)
    local a = ANCHOR[self.quadrantCode];
    --debug ("quadrant="..self.quadrantCode..", a="..a);
    GameTooltip:SetOwner(self, a);
    GameTooltip:SetText(L["TOOLTIP"]);
    GameTooltip:Show();
  end);

  self.target:SetScript("OnLeave", function (self)
    GameTooltip:Hide();
  end);

  self.target:SetScript("OnMouseUp", function (self, buttonName)
    --debug("clicked: "..buttonName);
    if (buttonName == "LeftButton") then
      -- remember where we are and how far we want to go.
      From.C, From.Z, From.X, From.Y = Astrolabe:GetCurrentPlayerPosition();
      local mx, my = GetCurrentCursorPosition();
      From.Distance = Astrolabe:ComputeDistance(From.C, From.Z, From.X, From.Y, From.C, From.Z, mx, my);
--      if CruiseControl.targetSet then
--        CruiseControl.targetSet = false;
--        say("disengaged", "FFb0d032");
--        playMySound("DESTCLEAR");
--      else
        CruiseControl.targetSet = true;
        CruiseControl.startedFlying = false;
	CruiseControl.approachAnnounced = false;
	CruiseControl.targetReached = false;
        say("engaged", "FFb0d032");
        playMySound("DESTSET");
--      end
    end
    if (buttonName == "RightButton") then
      say("disengaged", "FFb0d032");
      if (CruiseControl.targetSet) then
        playMySound("DESTCLEAR");
      end
      CruiseControl.targetSet = false;
      if CruiseControl.soundSettings then
        for k, v in pairs(CruiseControl.soundSettings) do
	  SetCVar(k, v);
	end
        CruiseControl.soundSettings = nil;
      end
    end
  end);

  msg("initialized.");
end

----------------------------------------------------------------------
-- DrawTarget
-- 

local function HideTarget(self)
  if self.target then
    self.target:Hide();
  end
end

local function DrawTarget(parent, px, py)
  -- target icon

  parent.target:ClearAllPoints();
  parent.target:SetPoint("CENTER", parent, "TOPLEFT", px, py);
  parent.target:Show();
end

---------------------------------------------------------------------
-- DrawLine
--
local Line;
local LINEFACTOR = 64/126; -- Multiplying factor for texture coordinates

local function HideLine()
  if Line then Line:Hide(); end
end

local function DrawLine(C, sx, sy, ex, ey)
	-- debug("DrawLine("..sx..", "..sy..", "..ex..", "..ey..")");

	local relPoint = "TOPLEFT"

	if (not Line) then
		Line = C:CreateTexture()
		if not Line:SetTexture("Interface\\AddOns\\CruiseControl\\line.tga") then
			error("DrawLine: SetTexture failed");
		end
		Line:SetDrawLayer("OVERLAY");
		Line:SetVertexColor(LINECOLOR[1], LINECOLOR[2], LINECOLOR[3], LINECOLOR[4]);
	end

	-- Determine dimensions and center point of line
	local dx,dy = ex - sx, ey - sy;

	-- Calculate actual length of line
	local l = math.sqrt((dx * dx) + (dy * dy));
	local w = LINEWIDTH;

	-- Quick escape if it's zero length
	if l == 0 then
		Line:ClearAllPoints();
		--Line:SetTexCoord(0,0,0,0,0,0,0,0);
		--Line:SetPoint("BOTTOMLEFT", C, relPoint, cx, cy);
		--Line:SetPoint("TOPRIGHT",   C, relPoint, cx, cy);
		Line:Hide()
		return Line;
	end

	local cx,cy = (sx + ex) / 2, (sy + ey) / 2;

	-- Normalize direction if necessary
	if (dx < 0) then
		dx,dy = -dx,-dy;
	end

	-- Sin and Cosine of rotation, and combination (for later)
	local s,c = -dy / l, dx / l;
	local sc = s * c;

	-- Calculate bounding box size and texture coordinates
	local Bwid, Bhgt, BLx, BLy, TLx, TLy, TRx, TRy, BRx, BRy;
	if (dy >= 0) then
		Bwid = ((l * c) - (w * s)) * LINEFACTOR;
		Bhgt = ((w * c) - (l * s)) * LINEFACTOR;
		BLx, BLy, BRy = (w / l) * sc, s * s, (l / w) * sc;
		BRx, TLx, TLy, TRx = 1 - BLy, BLy, 1 - BRy, 1 - BLx;
		TRy = BRx;
	else
		Bwid = ((l * c) + (w * s)) * LINEFACTOR;
		Bhgt = ((w * c) + (l * s)) * LINEFACTOR;
		BLx, BLy, BRx = s * s, -(l / w) * sc, 1 + (w / l) * sc;
		BRy, TLx, TLy, TRy = BLx, 1 - BRx, 1 - BLx, 1 - BLy;
		TRx = TLy;
	end

	-- Set texture coordinates and anchors
	Line:ClearAllPoints();
	Line:SetTexCoord( TLx, TLy, BLx, BLy, TRx, TRy, BRx, BRy )
	Line:SetPoint("BOTTOMLEFT", C, relPoint, cx - Bwid, cy - Bhgt);
	Line:SetPoint("TOPRIGHT",   C, relPoint, cx + Bwid, cy + Bhgt);
	Line:Show()
	return Line
end

----------------------------------------------------------------------
-- round coordinates to a certain precision
local coord_fmt = "%%.%df, %%.%df"
local function RoundCoords(x,y,prec)
  local fmt = coord_fmt:format(prec, prec)
  return fmt:format(x*100, y*100)
end

----------------------------------------------------------------------
-- main update function
CruiseControl:SetScript("OnUpdate", function (self, elapsed)

  self.timeSinceLastUpdate = self.timeSinceLastUpdate + elapsed;
	
  if self.timeSinceLastUpdate > UPDATE_INTERVAL then
    local C, Z, x, y = Astrolabe:GetCurrentPlayerPosition();
    local px, py = GetPlayerMapPosition("player");
    local ang = GetPlayerFacing();
    local dx, dy;
    local rx, ry;
    local mx, my = GetCurrentCursorPosition();
    local scale = WorldMapDetailFrame:GetEffectiveScale();

    local speed = GetUnitSpeed(self.currentUnit);
    local pitch = GetUnitPitch(self.currentUnit);
    if (speed and speed >= 0) then
      if (speed > MaxSpeed) then
        MaxSpeed = speed;
      end
      if (IsSwimming() or IsFlying()) then
        -- correct speed for Angle
        speed = speed * math.cos(pitch);
      end
    else
      speed = 0;
    end

    if (
        (px ~= PlayerX or py ~= PlayerY or Angle ~= ang)
	or (mx ~= MouseX or my ~= MouseY)
	and (px ~= 0 and py ~= 0)
	) then
      -- something changed, redraw
      PlayerX = px;
      PlayerY = py;
      Angle = ang;
      MouseX = mx;
      MouseY = my;
      width = self:GetWidth();
      height = self:GetHeight();
      linelength = width + height;
      py = -py;
      rx, ry = math.sin(ang) * -1, math.cos(ang);
      px = px * width;
      py = py * height;

      self.target.quadrantCode = (rx>0 and 1 or 0) + (ry>0 and 2 or 0) + 1;

      local kx;
      if (rx > 0) then
      	kx = (width - px)/rx;
      else
	kx = -px/rx;
      end

      local ky;
      if (ry < 0) then
      	ky = (-height - py)/ry;
      else
      	ky = -py/ry;
      end

      local k = kx;
      if math.abs(k) > math.abs(ky) then k = ky end

      if (px == 0 and py == 0) then 
        HideLine();
	HideTarget(self);
      else
        DrawLine(self, px, py, px + rx*k, py + ry*k);
	-- draw the target destination ring
        local target = "";
	local toa = "";
        if From.TOA > 0 then
          toa = string.format(" %d %s  ", From.TOA, L["sec"]);
	end
      	if(mx and my) then
      	  my = -my;
	  mx = mx * width;
	  my = my * height;
	  local oob = false;
	  local dist = (mx*rx + my*ry - rx*px - ry*py) / (rx*rx + ry*ry);
	  local qx, qy = px + dist * rx, py + dist * ry;
	  if (qx < 0 or qy > 0 or qx > width or qy < -height) then oob = true end

	  scaledFontSize = FONTSIZE / scale;
	  self.textString:SetFont(FONTSTYLE, scaledFontSize, FONTOUTLINE)

	  local yards = Astrolabe:ComputeDistance(C, Z, x, y, C, Z, MouseX, MouseY);
	  if (dist > 0 and not oob) then
	    DrawTarget(self, qx, qy);
	    yards = Astrolabe:ComputeDistance(C, Z, x, y, C, Z, qx/width, -qy/height);
	  else
	    HideTarget(self);
	  end
          target = string.format(" %s: %d %s  %s: %.1f,%.1f",
				 L["Target Dist"],
				 yards/MaxSpeed,
				 L["sec"],
				 L["Mouse"],
	    			 MouseX*100, MouseY*100
                                 );
        else
	  HideTarget(self);
	end
        scaledFontSize = FONTSIZE / scale;
	self.textString:SetFont(FONTSTYLE, scaledFontSize, FONTOUTLINE);
        self.textString:SetFormattedText("%s %s: %.1f/%s {%.1f, %.1f}%s%s  %s: %.0f",
                                         L["Speed"],
                                         L[self.currentUnit],
					 speed,
					 L["sec"],
				         x*100, y*100,
                                         toa,
                                         target,
                                         L["Ang"],
                                         math.sin(pitch) * 90
				        );
      end
    end
    
    self.timeSinceLastUpdate = 0;
  end
end);

----------------------------------------------------------------------
-- listener for vehicle change
-- register events

function CruiseControl:ADDON_LOADED(addon)
  if addon ~= "CruiseControl" then
    return;
  end
  self:UnregisterEvent("ADDON_LOADED");
  self:Init();
end

function CruiseControl:UNIT_ENTERED_VEHICLE(who)
  if who == "player" then
    CruiseControl.currentUnit = "vehicle";
  end
end

function CruiseControl:UNIT_EXITED_VEHICLE(who)
  if who == "player" then
    CruiseControl.currentUnit = "player";
  end
end

CruiseControl:SetScript("OnEvent", function(self, event, ...)
  self[event](self, ...);
end);

CruiseControl:RegisterEvent("UNIT_ENTERED_VEHICLE")
CruiseControl:RegisterEvent("UNIT_EXITED_VEHICLE")
CruiseControl:RegisterEvent("ADDON_LOADED")

